function [ind,xHOC,w,lb] = HOCbp(x,A,b,Sx,h,w0)
% function [ind,xHOC,w,lb] = HOCbp(x,A,b,Sx,h,w0)
% ==============================================================================
%  FILE:           HOCbp.m
%  VERSION:        1.00
%  AUTHOR:         Andreas M. Tillmann, TU Darmstadt, Germany
%  LAST MODIFIED:  09/28/2013
% ==============================================================================
%  DESCRIPTION:    Heuristic Optimality Check (HOC) for Basis Pursuit
%                      (BP)   min ||x||_1  s.t.  Ax=b.
%
%                  References:
%                  * Lorenz/Pfetsch/Tillmann, "Solving Basis Pursuit: Heuristic
%                  Optimality Check and Solver Comparison", 2011/12, preprint,
%                  http://www.optimization-online.org/DB_HTML/2011/07/3100.html
%                  * Tillmann, "Computational Aspects of Compressed Sensing", 
%                  submitted dissertation, 2013
%
%                  Copyright (C) Andreas M. Tillmann, 2013
%
%  INPUT:          x    The vector whose subdifferential is to be evaluated
%                  A    The coefficient matrix of the constraint Ax=b
%                  b    The corresponding right hand side vector; 
%                       it is assumed throughout that b is *not* all-zero
%                  Sx   Approx. support of vector x (index vector)
%                       [ optional; if argument Sx is not passed, then Sx
%                         will be index set w.r.t. threshold 1e-9 by default ]
%                  h    A subgradient of ||.||_1 at x
%                       [ optional; default is h = sign(x) ]
%                  w0   Vector of same size as Sx such that A(:,Sx)*w0 
%                       is initial guess for dual certificate
%                       (may be obtained from dual sol. est. w by A(:,Sx)\w)
%                       [ optional; default uses all-zero vector ]
%
%  OUTPUT:         ind  Optimality indicator, a value of true/1 indicates
%                       that the provided x is an optimal solution of (BP)
%                  xHOC If ind=false, this is just the input x, otherwise it
%                       is the (approx.) optimal vector, having support Sx
%                  w    The dual solution for which A'w is hopefully a
%                       subgradient of ||.||_1 at x
%                  lb   A lower bound on the optimal L1-value, obtained
%                       from w (scaled to be dual-feasible) as -b'*w
% ============================================================================
if( nargin < 4 )
    Sx = find(abs(x)>1e-9);  % default support approximation: ignore entries
end                          % with absolute value smaller than 1e-9
if( nargin < 5 )
    h = sign(x);             % default subgradient: sign-vector of x
end
if( nargin < 6 )
    w0 = spalloc(numel(Sx),1,numel(Sx)); % default: no dual information available
end

ind = false;  % optimality indicator
xHOC = x;

% calculate approx. (least-squares) solution to A(:,Sx)'w = -sign(x(Sx)) :
w = A(:,Sx)*CG( (A(:,Sx))', [], -h(Sx), 1e-9, 20, 1, w0 );
% --- the following works as well, but the corresp. system is larger:
%     w = CG( A(:,Sx), [], -A(:,Sx)'*h(Sx), 1e-9, 20, 1, zeros(size(b)) ); 
% --- here, CG only applies A(:,Sx) and A(:,Sx)', as specified by 1 in the 
%     2nd-to-last argument. If A(:,Sx)'*A(:,Sx) is explicitly available, 
%     this may be passed to CG (second argument) and is used in that code
%     if the 2nd-to-last argument is 0 (first argument may then be []).

% dual feasibility measure:
temp = norm(A'*w,'inf');
% --- unless b=0 in the original problem, an opt. sol. will always have at
%     least one nonzero entry, hence any subgradient to an optimal point 
%     must have at least one entry equal to +1 or -1.

% if dual feas., compute primal solution on support Sx and check optimality:
if( abs(temp) -1 < 1e-6 ) % allow small error/feas.-violation
    xHOC = sparse(Sx,1,A(:,Sx)\b,numel(x),1);
    if( norm(A*xHOC-b,'inf') < 1e-6 )     % i.e., xHOC is (approx.) feasible
        lb = -(b'*w)/temp;                % corresp. dual objective value
        xnrm = norm(xHOC,1);              % primal objective value
        if( abs( xnrm - lb )/xnrm < 1e-6) % small relative duality gap
            ind = true; % optimality condition is (approximately) satisfied
        end
    end
else
    if( nargout > 3 )      % compute dual lower bound on primal opt. value
        lb = -(b'*w)/temp; % w/temp is a feasible dual solution, even if it
    end                    % doesn't yield a subgradient A'w of ||x||_1
end
