function [ind,xHOC,lagr,y,gap] = HOCbpdn(x,A,b,delta,S,h)
%function [ind,xHOC,lagr,y,gap] = HOCbpdn(x,A,b,delta,S,h)
% ==============================================================================
%  FILE:           HOCbpdn.m
%  VERSION:        1.00
%  AUTHOR:         Andreas M. Tillmann, TU Darmstadt, Germany
%  LAST MODIFIED:  09/28/2013
% ==============================================================================
%  DESCRIPTION:    Heuristic Optimality Check (HOC) for Basis Pursuit Denoising
%                      (BPDN)   min ||x||_1  s.t.  ||Ax-b||_2 <= delta.
%
%                  Reference:
%                  * Tillmann, "Computational Aspects of Compressed Sensing", 
%                  submitted dissertation, 2013
%
%                  Copyright (C) Andreas M. Tillmann, 2013
%
%  INPUT:          x    The vector whose subdifferential is to be evaluated
%                  A    The coefficient matrix of the constraint 
%                  b    The corresponding right hand side vector; 
%                       it is assumed throughout that ||b||_2 > delta
%                  delta   Allowed deviation from Ax=b / noise level 
%                  S    Approx. support of vector x (index vector)
%                       [ optional; if argument S is not passed, then S
%                         will be index set w.r.t. threshold 1e-9 by default ]
%                  h    A subgradient of ||.||_1 at x
%                       [ optional; default is h = sign(x) ]
%
%  OUTPUT:         ind  Optimality indicator, a value of true/1 indicates
%                       that xHOC is an (approx.) optimal solution of (BPDN)
%                  xHOC Primal solution candidate having support S
%                  lagr   Lagrange multiplier assoc. to xHOC
%                  y    Dual solution 
%                  gap  relative duality gap for (xHOC,y)
% ============================================================================
if( nargin < 5 )
    S = find(abs(x)>1e-9);  % default support approximation: ignore entries
end                         % with absolute value smaller than 1e-9
if( nargin < 6 )
    h = sign(x);            % default subgradient: sign-vector of x
end

ind = false; % optimality indicator
sS = numel(S);
xHOC = spalloc(size(x,1),1,sS);

feastol = 1e-3;
opttol  = 1e-6;

AS = A(:,S);

% assert( rank(AS) = length(S) ); % assumed to hold true

u = AS'*b;
%ASTAS = AS'*AS;
%p = ASTAS\u;
s = h(S);
%q = ASTAS\s;
% compute (approx.) sol. qu to ASx'*ASx*q = u, and
% compute (approx.) sol. qs to ASx'*ASx*q = s :
qu = CG(AS',[],u,1e-9,25,1,spalloc(sS,1,sS));
qs = CG(AS',[],s,1e-9,25,1,spalloc(sS,1,sS));

% compute Lagrange multiplier estimate:
lagr = sqrt( ( norm(AS*qs)^2 ) / (delta^2 - norm(b)^2 + qu'*u) );

% compute primal solution:
xHOC(S) = qu - (1/lagr)*qs;

% compute dual solution:
y = lagr*(AS*xHOC(S)-b);

% verify primal and dual feasibility:
primfeas = ( abs( norm(A*xHOC-b) - delta ) <= feastol );
dualfeas = ( abs( norm(A'*y,'inf') - 1 )   <= feastol );

% check (relative) duality gap:
gap = abs( norm(xHOC(S),1) + b'*y + delta*norm(y) ) / norm(xHOC(S),1);
% = norm(xHOC,1) + b'*y + lagr*delta^2; % if xHOC exactly feasible
  
if primfeas && dualfeas && ( gap <= opttol )
    ind = true;
end
