function [ind,xHOC,y,gap] = HOCl1ls(x,A,b,lambda,S,h)
%function [ind,xHOC,y,gap] = HOCl1ls(x,A,b,lambda,S,h)
% ==============================================================================
%  FILE:           HOCl1ls.m
%  VERSION:        1.00
%  AUTHOR:         Andreas M. Tillmann, TU Darmstadt, Germany
%  LAST MODIFIED:  09/28/2013
% ==============================================================================
%  DESCRIPTION:    Heuristic Optimality Check (HOC) for Basis Pursuit Denoising
%                  in its original variant (L1-regularized least-squares)
%                      (L1LS)   min (1/2)*||A*x-b||_2^2 + lambda*||x||_1  
%
%                  Reference:
%                  * Tillmann, "Computational Aspects of Compressed Sensing", 
%                  submitted dissertation, 2013
%
%                  Copyright (C) Andreas M. Tillmann, 2013
%
%  INPUT:          x    The vector whose subdifferential is to be evaluated
%                  A    The coefficient matrix of the constraint 
%                  b    The corresponding right hand side vector 
%                  lambda  Regularization parameter
%                  S    Approx. support of vector x (index vector)
%                       [ optional; if argument S is not passed, then S
%                         will be index set w.r.t. threshold 1e-9 by default ]
%                  h    A subgradient of ||.||_1 at x
%                       [ optional; default is h = sign(x) ]
%
%  OUTPUT:         ind  Optimality indicator, a value of true/1 indicates
%                       that xHOC is an (approx.) optimal solution of (L1LS)
%                  xHOC Primal solution candidate having support S
%                  y    Dual solution 
%                  gap  relative duality gap for (xHOC,y)
% ============================================================================
if( nargin < 5 )
    S = find(abs(x)>1e-9);  % default support approximation: ignore entries
end                         % with absolute value smaller than 1e-9
if( nargin < 6 )
    h = sign(x);            % default subgradient: sign-vector of x
end

ind = false; % optimality indicator
sS = numel(S);
xHOC = spalloc(size(x,1),1,sS);

feastol  = 1e-3;
opttol  = 1e-6;

AS = A(:,S);

% assert( rank(AS) = length(S) ); % assumed to hold true

% primal solution: xHOC with zeros off support S and xHOC(S) is 
% computed as (approx.) sol. to AS'*AS*x = AS'*b - lambda*h(S):
xHOC(S) = CG(AS',[],AS'*b - lambda*h(S),1e-9,25,1,spalloc(sS,1,sS));
% no constraints in primal problem, so xHOC is always feasible

% dual solution:
y = AS*xHOC(S)-b;

% verify dual feasibility: 
dualfeas = ( abs( norm(A'*y,'inf') - lambda )  <= feastol );

% check (relative) duality gap:
gap = abs( norm(y)^2 + lambda*norm(xHOC(S),1) + b'*y ) / norm(xHOC(S),1);
% norm(y)^2 == 0.5*||AS*xHOC(S)-b||_2^2 + 0.5*||y||_2^2  % if exact

if dualfeas && ( gap <= opttol )
    ind = true;
end
